#!/bin/sh
# -----------------------------------------------------------------------------
# catalina.sh - Start/Stop Script for the CATALINA Server
#
# Environment Variable Prequisites
#
#   ANT_HOME      Must point at your Ant installation.
#   ANT_OPTS      (Optional) Java runtime options used when the "ant"
#                 command is executed.
#   DAV_HOME (Optional) May point at your Catalina "build" directory.
#                 If not present, the current working directory is assumed.
#   CATALINA_OPTS (Optional) Java runtime options used when the "start",
#                 "stop", or "run" command is executed.
#   CLASSPATH     Must contain the "jaxp.jar" and "parser.jar" files from
#                 the JAXP Reference Implementation, version 1.0, or these
#                 files must be in your Java2 extensions directory.
#   JAVA_HOME     Must point at your Java Development Kit installation.
#   SERVLET_JAR   (Optional) May point at a copy of the "servlet.jar" file
#                 to be used.  If not present, the file at
#                 "%TOMCAT_HOME%\lib\servlet.jar" will be assumed.
#   TOMCAT_HOME   (Optional) May point at your Tomcat "build" directory.
#                 If not present, the directory "%DAV_HOME%\..\tomcat"
#                 is assumed.
#
# $Id: dav.sh,v 1.3 2000/08/31 22:08:53 remm Exp $
# -----------------------------------------------------------------------------


# ----- Verify and Set Required Environment Variables -------------------------

if [ "$ANT_HOME" = "" ] ; then
  echo You must set ANT_HOME to point at your Ant installation
  exit 1
fi

if [ "$DAV_HOME" = "" ] ; then
  DAV_HOME=.
fi

if [ "$CATALINA_OPTS" = "" ] ; then
  CATALINA_OPTS=""
fi

if [ "$JAVA_HOME" = "" ] ; then
  echo You must set JAVA_HOME to point at your Java Development Kit installation
  exit 1
fi

if [ "$TOMCAT_HOME" = "" ] ; then
  TOMCAT_HOME=$DAV_HOME/../tomcat
fi

if [ "$SERVLET_JAR" = "" ] ; then
  SERVLET_JAR=$TOMCAT_HOME/lib/servlet.jar
fi


# ----- Set Up The Runtime Classpath

if [ "$CLASSPATH" = "" ] ; then
  CP=$TOMCAT_HOME/classes
else
  CP=$TOMCAT_HOME/classes:$CLASSPATH
fi
for i in $DAV_HOME/lib/*.jar ; do
  CP=$CP:$i
done
echo Using CLASSPATH: $CP


# ----- Execute The Requested Command -----------------------------------------

if [ "$1" = "env" ] ; then

  export DAV_HOME CP SERVLET_JAR TOMCAT_HOME
  exit 0

elif [ "$1" = "run" ] ; then

  java $CATALINA_OPTS -classpath $CP -Dcatalina.home=$DAV_HOME org.apache.catalina.startup.Catalina "$@" start

elif [ "$1" = "start" ] ; then

  java $CATALINA_OPTS -classpath $CP -Dcatalina.home=$DAV_HOME org.apache.catalina.startup.Catalina "$@" start &

elif [ "$1" = "stop" ] ; then

  java $CATALINA_OPTS -classpath $CP -Dcatalina.home=$DAV_HOME org.apache.catalina.startup.Catalina "$@" stop

else

  echo "Usage: catalina.sh ( env | run | start | stop)"
  echo "Commands:"
  echo "  env -   Set up environment variables that Catalina would use"
  echo "  run -   Start Catalina in the current window"
  echo "  start - Start Catalina in a separate window"
  echo "  stop -  Stop Catalina"
  exit 1

fi
