/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/LoginFrame.java,v 1.1.1.1 2001/05/15 06:04:38 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:38 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.Keymap;

import org.apache.commons.httpclient.*;
import org.apache.webdav.lib.*;
import org.apache.webdav.ui.filechooser.*;
import org.apache.webdav.ui.filechooser.dir.*;

/**
 * Title:        LoginFrame.java
 * Description:  A Login Dialog box
 *
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class LoginFrame extends JDialog {
  JPanel contentPane;
  BorderLayout contentPaneLayout = new BorderLayout();
  JPanel SessionPane = new JPanel();
  GridBagLayout sessionPaneLayout = new GridBagLayout();
  JPanel emptyPanel1 = new JPanel();
  JLabel hostLabel = new JLabel();
  JTextField hostField = new JTextField();
  JLabel portLabel = new JLabel();
  JTextField portField = new JTextField();
  JPanel emptyPanel2 = new JPanel();
  JLabel userLabel = new JLabel();
  JTextField userField = new JTextField();
  JLabel passwdLabel = new JLabel();
  JPasswordField passwdField = new JPasswordField();
  JButton resetButton = new JButton();
  JButton okButton = new JButton();
  JPanel emptyPanel4 = new JPanel();
  JPanel emptyPanel3 = new JPanel();

  private boolean focusIsSet = false;
  private String port;
  private String host;
  private String userName;
  private String userPasswd;
  private String rootPath;
  private SPDirectoryService directoryService;
  private HttpClient webDavClient=null;
  private SPFileChooser webFileChooser=null;
  private boolean proxy = false;

  public LoginFrame(Frame owner, String webDavServer, String rootPath,
              String userName, String userPasswd) {
    super(owner, true);
    int colonIndex = webDavServer.lastIndexOf(":");
    this.host = webDavServer.substring(0, colonIndex);
    this.port = webDavServer.substring(colonIndex + 1);
    this.rootPath = rootPath;
    this.userName = userName;
    this.userPasswd = userPasswd;

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void showAtCentre()  {
    this.validate();

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = this.getSize();
    if (frameSize.height > screenSize.height) {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
      frameSize.width = screenSize.width;
    }
    this.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    this.setVisible(true);

  }

  public HttpClient getWebdavClient() {
    return webDavClient;
  }

  public SPFileChooser getWebFileChooser()  {
    return webFileChooser;
  }

  public SPDirectoryService getDirectoryService() {
    return webFileChooser.getDirectoryService();
 }

  void resetTextFields()  {
    hostField.setText(host);
    portField.setText(port);
    userField.setText("");
    passwdField.setText("");
    focusIsSet = false;
    setFocus();
  }

  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);

    if (e.getID() == WindowEvent.WINDOW_ACTIVATED) {
      this.setFocus();
    }

  }

  /**Component initialization*/
  private void jbInit() throws Exception  {
    //setIconImage(Toolkit.getDefaultToolkit().createImage(LoginFrame.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(contentPaneLayout);
    this.setSize(new Dimension(400, 300));
    this.setTitle("Host Connection");
    SessionPane.setBorder(BorderFactory.createRaisedBevelBorder());
    SessionPane.setLayout(sessionPaneLayout);
    emptyPanel1.setLayout(null);
    hostLabel.setMaximumSize(new Dimension(100, 17));
    hostLabel.setMinimumSize(new Dimension(100, 17));
    hostLabel.setPreferredSize(new Dimension(100, 17));
    hostLabel.setRequestFocusEnabled(false);
    hostLabel.setHorizontalAlignment(SwingConstants.TRAILING);
    hostLabel.setText("Host Name:");
    hostField.setMinimumSize(new Dimension(260, 21));
    hostField.setPreferredSize(new Dimension(280, 21));
    hostField.setColumns(50);
    hostField.setText(host);
    emptyPanel1.setMinimumSize(new Dimension(100, 51));
    emptyPanel1.setPreferredSize(new Dimension(100, 51));
    emptyPanel1.setRequestFocusEnabled(false);
    portLabel.setMaximumSize(new Dimension(100, 17));
    portLabel.setMinimumSize(new Dimension(100, 17));
    portLabel.setPreferredSize(new Dimension(100, 17));
    portLabel.setRequestFocusEnabled(false);
    portLabel.setHorizontalAlignment(SwingConstants.TRAILING);
    portLabel.setText("Port:");
    portField.setMinimumSize(new Dimension(60, 21));
    portField.setPreferredSize(new Dimension(60, 21));
    portField.setText(port);
    emptyPanel2.setMinimumSize(new Dimension(100, 51));
    emptyPanel2.setPreferredSize(new Dimension(100, 51));
    emptyPanel2.setRequestFocusEnabled(false);
    emptyPanel2.setLayout(null);
    userLabel.setMaximumSize(new Dimension(100, 17));
    userLabel.setMinimumSize(new Dimension(100, 17));
    userLabel.setPreferredSize(new Dimension(100, 17));
    userLabel.setRequestFocusEnabled(false);
    userLabel.setHorizontalAlignment(SwingConstants.TRAILING);
    userLabel.setText("User Name:");
    passwdLabel.setMaximumSize(new Dimension(100, 17));
    passwdLabel.setMinimumSize(new Dimension(100, 17));
    passwdLabel.setPreferredSize(new Dimension(100, 17));
    passwdLabel.setRequestFocusEnabled(false);
    passwdLabel.setHorizontalAlignment(SwingConstants.TRAILING);
    passwdLabel.setText("Password:");
    resetButton.setRequestFocusEnabled(false);
    resetButton.setText("RESET");
    resetButton.addActionListener(new LoginFrame_resetButton_actionAdapter(this));
    okButton.setRequestFocusEnabled(false);
    okButton.setText("OK");
    okButton.addActionListener(new LoginFrame_okButton_actionAdapter(this));
    emptyPanel3.setLayout(null);
    emptyPanel3.setRequestFocusEnabled(false);
    emptyPanel3.setPreferredSize(new Dimension(100, 51));
    emptyPanel3.setMinimumSize(new Dimension(100, 51));
    userField.setMinimumSize(new Dimension(80, 21));
    userField.setPreferredSize(new Dimension(80, 21));
    userField.setText(userName);
    passwdField.setPreferredSize(new Dimension(80, 21));
    passwdField.setMinimumSize(new Dimension(80, 21));
    passwdField.setText(userPasswd);
    contentPane.add(SessionPane, BorderLayout.CENTER);
    SessionPane.add(emptyPanel1, new GridBagConstraints(0, 0, 4, 1, 0, 0,
            GridBagConstraints.SOUTH, GridBagConstraints.BOTH,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(emptyPanel2, new GridBagConstraints(0, 3, 4, 1, 0, 0,
            GridBagConstraints.SOUTH, GridBagConstraints.BOTH,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(userLabel, new GridBagConstraints(0, 4, 1, 1, 0, 0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(passwdLabel, new GridBagConstraints(0, 5, 1, 1, 0, 0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(resetButton, new GridBagConstraints(0, 7, 1, 1, 0, 0,
            GridBagConstraints.WEST, GridBagConstraints.NONE,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(okButton, new GridBagConstraints(3, 7,
	    GridBagConstraints.REMAINDER, 1, 0, 0,
            GridBagConstraints.EAST, GridBagConstraints.NONE,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(portLabel, new GridBagConstraints(0, 2, 1, 1, 0, 0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(hostLabel, new GridBagConstraints(0, 1, 1, 1, 0, 0,
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(hostField, new GridBagConstraints(1, 1, 3, 1, 0, 0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(emptyPanel3, new GridBagConstraints(0, 6, 4, 1, 0, 0,
            GridBagConstraints.SOUTH, GridBagConstraints.BOTH,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(portField, new GridBagConstraints(1, 2, 1, 1, 0, 0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 20), 0, 0));
    SessionPane.add(passwdField, new GridBagConstraints(1, 5, 1, 1, 0, 0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));
    SessionPane.add(userField, new GridBagConstraints(1, 4, 1, 1, 0, 0,
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
	    new Insets(0, 0, 0, 0), 0, 0));

  /* Remove the RETURN key from the shared map of JTextFields */
    Keymap map = hostField.getKeymap();
    KeyStroke key = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
    map.removeKeyStrokeBinding(key);

  /* Make the OK button the default button */
    this.getRootPane().setDefaultButton(okButton);

  } //private jbInit()

  private void setFocus() {
    if (!focusIsSet) {
       passwdField.requestFocus();
       focusIsSet = true;
    }
  }

  private HttpClient createWebdavClient() {
    host = hostField.getText();
    port = portField.getText();
    userName = userField.getText();
    userPasswd = new String(passwdField.getPassword());

    int portNumber = Integer.parseInt(portField.getText());
    String proxyServer;
    int proxyPort;

    HttpClient client = new HttpClient();

    if (proxy) {
        proxyServer = "";
        int colonIndex = proxyServer.indexOf(':');
        proxyPort = Integer.parseInt(proxyServer.substring(colonIndex+1));
        proxyServer = proxyServer.substring(0,colonIndex);
        //client.startSession(host, portNumber, proxyServer, proxyPort);
    } else  {
        client.startSession(host, portNumber);
    }

    // Set the credentials to use - see org.apache.webdav.lib.Credentials
    Credentials credentials = new Credentials(userName, userPasswd);
    client.setCredentials(credentials);

    return client;
  }

  protected void resetButton_actionPerformed(ActionEvent e) {
    resetTextFields();
  }

  protected void okButton_actionPerformed(ActionEvent e) {
    try {
      webDavClient = createWebdavClient();
      webFileChooser = SPFileChooser.createWebFileChooser(webDavClient, rootPath);
      setVisible(false);
    } catch (Exception exception) {
      final String title = "Connection Failure";
      final String msg = "Unable to connect to "+host+":"
	    +portField.getText()+"\n"+exception.getMessage();
      JOptionPane.showMessageDialog(null, msg,
                title, JOptionPane.ERROR_MESSAGE);
      exception.printStackTrace();
    }
  }

} // End of LoginFrame class

class LoginFrame_resetButton_actionAdapter implements java.awt.event.ActionListener {
  LoginFrame adaptee;

  LoginFrame_resetButton_actionAdapter(LoginFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.resetButton_actionPerformed(e);
  }
} // End of LoginFrame_resetButton_actionAdapter class

class LoginFrame_okButton_actionAdapter implements java.awt.event.ActionListener {
  LoginFrame adaptee;

  LoginFrame_okButton_actionAdapter(LoginFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.okButton_actionPerformed(e);
  }
} // End of LoginFrame_okButton_actionAdapter class
