/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/SPFileSystemView.java,v 1.1.1.1 2001/05/15 06:04:39 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:39 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.filechooser.*;

import org.apache.webdav.ui.filechooser.dir.*;

/**
 * Title:        SPFileSystemView.java
 * Description:  SPFileSystemView is the SPFileChooser's gateway to the
 *               file system.
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPFileSystemView extends FileSystemView {

    private SPDirectoryService directoryService;

    public SPFileSystemView(SPDirectoryService    aDirectoryService){
        this.directoryService = aDirectoryService;
    }

    /*
     * Creates a new folder with a default folder name.
     * This method calls into the currently opened
     * directory service to obtain the directory service
     * system specific way to create a new folder.
     */
    public File createNewFolder(File aContainingDir)
                              throws IOException {
        File f = directoryService.createNewFolder(aContainingDir);
        return f;
    }

    /*
     * Returns a File object constructed from the given path string.
     */
    public File createFileObject(String path) {
        return new SPFile(path, directoryService);
    }

    /*
     * Returns a File object constructed in dir from the given filename.
     */
    public File createFileObject(File dir, String filename) {
        if(dir == null) {
          return new SPFile(filename, directoryService);
        } else {
          return new SPFile(dir, filename, directoryService);
        }
    }

    /*
     * Returns all root partitians on this system.
     */
    public File[] getRoots() {
        File[] arrFiles = new File[1];
        arrFiles[0] = directoryService.getRoot();
        return arrFiles;
    }

    /*
     * Returns whether a file is hidden or not.
     * Currently, there is no concept of
     * a hidden file.
     */
    public boolean isHiddenFile(File f) {
        return false;
    }

    /*
     * Determines if the file is a root partition.
     */
    public boolean isRoot(File f) {
        File root = directoryService.getRoot();
        boolean b = root.equals(f);
        return b;
    }

    /*
     * Returns the WebFolder Root directory.
     */
    public File getHomeDirectory() {
        return directoryService.getRoot();
    }

    /*
     * Returns the parent directory of dir.
     */
    public File getParentDirectory(File dir) {
	if(dir != null) {
	    File f = createFileObject(dir.getAbsolutePath());
	    String parentFilename = f.getParent();
	    if(parentFilename != null) {
		return new SPFile(parentFilename, directoryService);
	    }
	}
	return null;
    }

    /*
     * Gets the list of files. Invoked internally
     * by the file chooser when the directory
     * is changed.
     */
    public File[] getFiles(File dir,  boolean useFileHiding){
        File[] arrFiles = directoryService.getChildren(dir);
        return arrFiles;
    }

} // End of SPFileSystemView class

