/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPDirectoryService.java,v 1.1.1.1 2001/05/15 06:04:40 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:40 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import javax.swing.*;
import java.io.*;
import java.util.*;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;

import org.apache.webdav.lib.*;
import org.apache.webdav.ui.lib.methods.SPPropFindMethod;
import org.apache.webdav.ui.util.SPUtils;

/**
 * Title:        SPDirectoryService.java
 * Description:  Provides services which are related to directory queries to
 *               SPFileChooser, SPFile, SPFileView, SPFileSystemView,
 *               and SPFileFilter
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPDirectoryService {

  /*
   * A hashtable whose keys are File's path String object and
   * values are the File's corresponding SPResourceNode object
   */
  private HashMap hashmap = new HashMap();

  HttpClient client;
  SPResourceNode rootAsNode = null;
  File rootAsFile = null;

  public SPDirectoryService(HttpClient client, String rootUrlPath)
      throws Exception {
    this.client = client;
    SPWebFolder root = new SPWebFolder(client, rootUrlPath);

    try {
      root.init();
      if (System.getProperty("os.name").indexOf("Windows")>=0)  {
      	rootAsFile = new SPFile("C:\\", root.getName(), this);
      } else  {
      	rootAsFile = new SPFile("/",root.getName(), this);
      }

      rootAsNode = (SPResourceNode) root;
      this.hashmap.put(rootAsFile.getPath(), rootAsNode);
    }
    catch(Exception e) {
      e.printStackTrace();
      throw e;
    }
  }


  /*
   * Returns the children of the parent directory.
   */
  public File[] getChildren(File aDir)  {

    File[] arrFiles = null;
    SPResourceNode node = getSPResourceNode(aDir);

    if (node != null) {
      ArrayList children = node.getChildren();

      if (children != null)  {
        arrFiles = new File[children.size()];
        int k = 0;
        for (Iterator i=children.iterator(); i.hasNext();)  {
          SPResourceNode childNode = (SPResourceNode) i.next();
          File f = new SPFile(aDir, childNode.getName(), this);
          arrFiles[k++] = f;
          this.hashmap.put(f.getPath(), childNode);
        }
      } // if (children != null)  {
    }//if (node != null) {

    return arrFiles;
  }

  /*
   * Returns the root.
   */
  public File getRoot()  {
    return rootAsFile;
  }

  /*
   * Returns whether the file is traversable.
   */
  public boolean isTraversable(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);

    if (node == null) {
      return false;
    }
    return node.isTraversable();
  }

  /*
   * Returns whether the file is a directory.
   */
  public boolean isDirectory(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);

    if (node == null) {
      return false;
    }

    return node.isWebFolder();
  }

  /*
   * Returns whether the file is a file (not a directory).
   */
  public boolean isFile(File aFile)  {

    SPResourceNode node = getSPResourceNode(aFile);

    if (node == null) {
      return false;
    }

    return (! node.isWebFolder());
  }

  /*
   * Returns whether the file is locked.
   * NOTE : The file is locked when it is locked
   *        by other users OR by the current user.
   */
  public boolean isLocked(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.isLocked();
  }

  public boolean isReadOnly(File aFile) {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.isReadOnly();
  }

  public boolean hasReadAccess(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.hasReadAccess();
  }

  public boolean hasWriteAccess(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.hasWriteAccess();
  }

  public boolean hasReadWriteAccess(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.hasReadWriteAccess();
  }

  /*
   * Creates a new folder in the containing directory.
   */
  public File createNewFolder(File aContainingDir) {
    final String title = "New Folder Error";
    final String msg = "Service is not available yet.";
    JOptionPane.showMessageDialog(null, msg,
                title, JOptionPane.ERROR_MESSAGE);
    return null;
  }

  /*
   * Returns the type (i.e : Collection or Non-Collection).
   */
  public int getType(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    return node.getType();
  }

  public InputStream getWebFileContentAsStream(File webFile) throws Exception {
    return getWebFileContentAsStream(webFile, false);
  }

  public InputStream getWebFileContentAsStream(File webFile,
                                           boolean writeLock) throws Exception {
    SPWebFile node = (SPWebFile) getSPResourceNode(webFile);
    if (node == null) {
      return null;
    }

    return (node.getFileContentAsStream(writeLock));
  }

  public InputStream getWebFileContentAsStream(File webFile,
                             boolean writeLock, long timeout) throws Exception {
    SPWebFile node = (SPWebFile) getSPResourceNode(webFile);
    if (node == null) {
      return null;
    }

    return (node.getFileContentAsStream(writeLock, timeout));
  }

  public String getWebFileContentAsString(File webFile) throws Exception {
    return getWebFileContentAsString(webFile, false);
  }

  public String getWebFileContentAsString(File webFile,
                                           boolean writeLock) throws Exception {
    SPWebFile node = (SPWebFile) getSPResourceNode(webFile);
    if (node == null) {
      return null;
    }

    return (node.getFileContentAsString(writeLock));
  }

  public String getWebFileContentAsString(File webFile,
                             boolean writeLock, long timeout) throws Exception {
    SPWebFile node = (SPWebFile) getSPResourceNode(webFile);
    if (node == null) {
      return null;
    }

    return (node.getFileContentAsString(writeLock, timeout));
  }

  public int putWebFileContent(File aFile, String fileContent) throws Exception  {
    int statusCode;

    SPWebFile node = (SPWebFile) getSPResourceNode(aFile);

    if (node == null) { //new file
      //get its parent. NOTE: parent node always exists.
      SPWebFolder parentDir = (SPWebFolder) getSPResourceNode(aFile.getParentFile());
      String parentUrlPath = parentDir.getResourceUrlPath();

      if (parentDir.hasWriteAccess())   {
          node = new SPWebFile(client, parentUrlPath+aFile.getName());
          statusCode = node.putFileContent(fileContent);
          if ( statusCode == HttpStatus.SC_OK) {
              parentDir.addWebFile(node);
              this.hashmap.put(aFile.getPath(), node);
          }
      } else {
          String title = "Export Failure";
          String msg = "Write Access Denied in the Parent Directory ("
                +HttpStatus.SC_FORBIDDEN+")\n"
                +"Resource Path = "+parentUrlPath;
          SPUtils.showMessageDialog(title, msg);
          statusCode = HttpStatus.SC_FORBIDDEN;
      }
    } else  {
      statusCode = node.putFileContent(fileContent);
    }

    return statusCode;

  }


  /*
   * Determines if the file should be displayed
   * based on the current filter.
   */
  public boolean accept(File aFile, String aFilterDescription)  {
   /* I'd rather filter the SPResourceNode(s) found in 'hashmap'
    * because File.getName() may contain the resource node's 'displayName'
    * property.
    */

    boolean isAcceptable = false;
    SPResourceNode node = getSPResourceNode(aFile);

    if (node != null) {
      isAcceptable = node.isAcceptable(aFilterDescription);
    }

    return isAcceptable;

  }

  /*
   * Locks the specified file
   *
   * @params aFile a File to be locked
   * @returns a lock token
   */
  public String lock(File aFile)  {
    SPResourceNode node = getSPResourceNode(aFile);
    if (node != null) {
      return node.lock();
    }
    return null;
  }

  /*
   * Locks the specified file for 'timeout' seconds
   *
   * @params aFile a File to be locked
   * @returns a lock token
   */
  public String lock(File aFile, long timeout)  {
    SPResourceNode node = getSPResourceNode(aFile);
    if (node != null) {
      return node.lock(timeout);
    }
    return null;
  }

  /*
   * UnLocks the specified file
   *
   * @params aFile a File to be unlocked
   * @returns true when succesfull, false otherwise
   */
  public boolean unlock(File aFile)  {
    boolean theFlag = true;
    SPResourceNode node = getSPResourceNode(aFile);
    if (node != null) {
      theFlag = node.unlock();
    }
    return theFlag;
  }

  /*
   * Check whether the specified file exists in the hashmap
   */
  public boolean exists(File aFile) {
    return this.hashmap.containsKey(aFile.getPath());
  }

  /*
   * Returns the associated SPResourceNode in 'hashmap'
   */
  private SPResourceNode getSPResourceNode(File aFile)  {
    SPResourceNode node = (SPResourceNode) this.hashmap.get(aFile.getPath());
    return node;
 }

}// End of SPDirectoryService class
