/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/dir/SPResourceNode.java,v 1.1.1.1 2001/05/15 06:04:41 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:41 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser.dir;

import java.util.ArrayList;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;

import org.apache.webdav.lib.properties.*;
import org.apache.webdav.lib.*;
import org.apache.webdav.ui.util.SPUtils;

/**
 * Title:        SPResourceNode.java
 * Description:  An abstract Resource Node class
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public abstract class SPResourceNode {

  public static int COLLECTION_TYPE = 1, NON_COLLECTION_TYPE = 2,
             READ_ONLY = 3, READ_WRITE = 4, DEFAULT_TIMEOUT=300;

  protected HttpClient client;
  protected String resourceUrlPath;
  protected String name=null;
  protected boolean readAccess=false, writeAccess=false,
                    accessPrivilegeChecked=false;

  public abstract ArrayList getChildren();

  public String getName() {
    return name;
  }

  public String getResourceUrlPath() {
    return resourceUrlPath;
  }

  public abstract int getType();

  public void setName(String newName)  {
    this.name = newName;
  }

  public boolean isTraversable()  {
    return isWebFolder();
  }

  public abstract boolean isWebFolder();

  public abstract boolean isAcceptable(String aFilterDescription);

  public boolean isLocked() {
    return SPUtils.isLocked(this);
  }

  public boolean isReadOnly() {
    /* A node is read only if the resource is being locked
     * by other users
     */
    boolean lockedByOthers = false;

    String lockToken = SPUtils.getLockToken(client, resourceUrlPath);
    if ((lockToken == null) && (isLocked()))  {
      lockedByOthers = true;
    }

    return lockedByOthers;
  }

  public String lock()  {
    return SPUtils.lock(this);
  }

  public String lock(long timeout)  {
    return SPUtils.lock(this, timeout);
  }

  public boolean unlock()  {
    return SPUtils.unlock(this);
  }

  public Property getProperty(String propertyName)  {
    return SPUtils.getProperty(this, propertyName);
  }

  public HttpClient getClient() {
    return client;
  }

  public String getLockToken()  {
    return SPUtils.getLockToken(client, resourceUrlPath);
  }

  public boolean hasReadAccess()  {
    if (! accessPrivilegeChecked)  {
        checkAccessPrivilege();
    }
    return readAccess;
  }

  public boolean hasWriteAccess()  {
    if (! accessPrivilegeChecked)  {
        checkAccessPrivilege();
    }
    return writeAccess;
  }

  public boolean hasReadWriteAccess()  {
    if (! accessPrivilegeChecked)  {
        checkAccessPrivilege();
    }
    return readAccess && writeAccess;
  }

  /*
   * Set readAccess and writeAccess fields to that set by the server
   */
  protected void checkAccessPrivilege()  {
    CurrentUserPrivilegeSetProperty userPrivilege = (CurrentUserPrivilegeSetProperty)
            SPUtils.getProperty(client, resourceUrlPath, "current-user-privilege-set");
    if (userPrivilege != null)  {
        readAccess = userPrivilege.hasReadAccess();
        writeAccess = userPrivilege.hasWriteAccess();
        accessPrivilegeChecked = true;
    } else  {
        accessPrivilegeChecked = false;
    }
  }
} //End of SPResourceNode class


