/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lib/methods/SPLockMethod.java,v 1.1.1.1 2001/05/15 06:04:46 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:46 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lib.methods;

import java.util.Enumeration;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.webdav.lib.*;
import org.apache.webdav.lib.methods.*;
import org.apache.webdav.lib.properties.*;
import org.apache.webdav.ui.filechooser.dir.SPResourceNode;
import org.apache.webdav.ui.util.SPUtils;

public class SPLockMethod extends LockMethod
      implements SPWebdavMethod {

  HttpClient client;

  public SPLockMethod(HttpClient client)  {
    this.client = client;
    setOwner(client.getCredentials().getUserName());
    setTimeout(SPResourceNode.DEFAULT_TIMEOUT);
  }

  public SPLockMethod(HttpClient client, String uri, long timeout) {
    this.client=client;
    setOwner(client.getCredentials().getUserName());
    setPath(uri);
    setTimeout(timeout);
  }

  public SPLockMethod(HttpClient client, String uri,
                    String refreshOpaqueToken, long timeout) {
    super(uri, refreshOpaqueToken, timeout);
    this.client = client;
    setOwner(client.getCredentials().getUserName());
  }

  public SPLockMethod(SPResourceNode resource, long timeout)  {
    this.client = resource.getClient();
    setOwner(client.getCredentials().getUserName());
    setPath(resource.getResourceUrlPath());
    setTimeout(timeout);
  }

  public SPLockMethod(SPResourceNode resource,
                    String refreshOpaqueToken, long timeout) {
    super(resource.getResourceUrlPath(), refreshOpaqueToken, timeout);
    this.client = client;
    setOwner(client.getCredentials().getUserName());
  }

  public void setClient(HttpClient client)  {
    this.client = client;
    setOwner(client.getCredentials().getUserName());
  }

  public HttpClient getClient() {
    return client;
  }

  public String getMessageContent() {
    String theString = new String();
    Enumeration responses = getResponses();

    while (responses.hasMoreElements()) {
      theString.concat((responses.nextElement()).toString()+"\n");
    }

    return theString;
  }

  public void refresh() {
    if (hasBeenUsed())  {
      recycle();
    }
  }

  public SPWebdavMethod execute() throws Exception {
    String lockToken = SPUtils.getLockToken(client, path);
    SPLockMethod executedLockMethod = this;
    try {
      if (lockToken != null) {
        executedLockMethod = refresh(lockToken);
      } else  {
        client.executeMethod(this.getWebdavMethodHandle());
      }
    } catch(Exception e) {
      e.printStackTrace();
      throw e;
    }
    return executedLockMethod;
  }//execute()

  public HttpMethod getWebdavMethodHandle() {
    return this;
  }

  public String toString()	{
    return (new String("LockMethod()"));
  }

  private SPLockMethod refresh(String lockToken) throws Exception {
    SPLockMethod lockMethod = new SPLockMethod(client, path, lockToken, getTimeout());
    try {
      client.executeMethod(lockMethod.getWebdavMethodHandle());
    } catch(Exception e)  {
      throw e;
    }
    return lockMethod;
  }

}//End of SPLockMethod class







