/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lib/methods/SPPropFindMethod.java,v 1.1.1.1 2001/05/15 06:04:48 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:48 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lib.methods;

import java.util.*;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.webdav.lib.*;
import org.apache.webdav.lib.methods.*;
import org.apache.webdav.lib.properties.*;
import org.apache.webdav.ui.filechooser.dir.SPResourceNode;

public class SPPropFindMethod extends PropFindMethod
      implements SPWebdavMethod {

  private int default_DEPTH = 1;
  private int default_PROP_TYPE = PropFindMethod.ALL;

  HttpClient client;

  public SPPropFindMethod(HttpClient client)  {
    this.client = client;
    setDepth(default_DEPTH);
    setType(default_PROP_TYPE);
  }

  public SPPropFindMethod(SPResourceNode resource)  {
    this.client = resource.getClient();
    setPath(resource.getResourceUrlPath());
    setDepth(default_DEPTH);
    setType(default_PROP_TYPE);
  }

  public void setClient(HttpClient client)  {
    this.client = client;
  }

  public HttpClient getClient() {
    return client;
  }

  public String getMessageContent()   {
    return ( getAllResponseURLs_asString() + "\n\n" +

             getResponseProperties_asString(getPath()) );
  }

  private String getAllResponseURLs_asString() {
    String allResponseURLs = new String("\nALL RESPONSE URLs: \n");
    Enumeration urls = getAllResponseURLs();

    while (urls.hasMoreElements())  {
      String s = (String) urls.nextElement();
      allResponseURLs = allResponseURLs.concat(s+"\n");
    }

    return allResponseURLs;

  }

  private String getResponseProperties_asString(String urlPath) {
    String responseProperties = new String("RESPONSE PROPERTIES of "+urlPath+" : \n");
    Enumeration props = getResponseProperties(urlPath);
    String stringLine = "";

    while (props.hasMoreElements())  {
      Property s = (Property) props.nextElement();
      stringLine = s.getName()+" = "+s.getPropertyAsString()+"\n";

      responseProperties = responseProperties.concat(stringLine);
    }

    return responseProperties;
  }

  public Property getSPProperty(String propertyName) {
    return getSPProperty(propertyName, this.path);
  }

  public Property getSPProperty(String propertyName, String urlPath) {
    Property theSPProperty = null;

    Enumeration propEnum = getResponseProperties(urlPath);
    boolean isFound = false;

    while ( !isFound && propEnum.hasMoreElements()){
        Property prop = (Property) propEnum.nextElement();
        if ( (prop.getName()).lastIndexOf(propertyName) != -1 ) {
          theSPProperty = prop;
          isFound = true;
        }
    }// while (enum...)

    return theSPProperty;
  }

  public boolean isCollection(String url)  {
    boolean resourcePropIsFound = false;
    boolean collection = false;
    ResourceTypeProperty resourceTypeProperty = null;

    Enumeration propEnum = getResponseProperties(url);
    while ( !resourcePropIsFound && propEnum.hasMoreElements()){
        Object object = propEnum.nextElement();
        Property prop = (Property) object;
        if ( (prop.getName()).lastIndexOf("resourcetype") != -1 ) {
          resourceTypeProperty = (ResourceTypeProperty) object;
          resourcePropIsFound = true;
        }
    }// while (enum...)

    if (resourceTypeProperty != null)  {
      collection = resourceTypeProperty.isCollection();
    }

    return collection;

  } // isCollection()

  public void refresh() {
    if (hasBeenUsed())  {
      recycle();
    }
  }

  public SPWebdavMethod execute() throws Exception {

    try {
      client.executeMethod(this.getWebdavMethodHandle());
    }
    catch(Exception e) {
      e.printStackTrace();
      throw e;
    }
    return this;
  }

  public HttpMethod getWebdavMethodHandle() {
    return this;
  }

  public String toString()	{
    return (new String("PropFindMethod()"));
  }

}// End of SPPropFindMethod class






