/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/lock/NodeLock.java,v 1.8 2001/02/15 17:45:49 remm Exp $
 * $Revision: 1.8 $
 * $Date: 2001/02/15 17:45:49 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.lock;

import java.util.Date;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.util.MD5Encoder;
import org.apache.slide.util.Messages;

/**
 * NodeLock class.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.8 $
 */
public final class NodeLock implements Cloneable, java.io.Serializable {
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Constructor.
     */
    public NodeLock() {
    }
    
    
    /**
     * Constructor.
     *
     * @param locked Locked object
     * @param user Lock owner
     * @param lockType Type of the lock
     * @param expirationDate Date of expiration of the lock
     * @param inheritance True if lock is inheritable
     */
    public NodeLock(ObjectNode locked, SubjectNode user, ActionNode lockType,
                    Date expirationDate, boolean inheritance) {
        this(locked.getUri(), user.getUri(), lockType.getUri(), expirationDate,
             inheritance);
    }
    
    
    /**
     * Constructor.
     *
     * @param locked Locked object
     * @param user Lock owner
     * @param lockType Type of the lock
     * @param expirationDate Date of expiration of the lock
     * @param inheritance True if lock is inheritable
     */
    public NodeLock(ObjectNode locked, SubjectNode user, ActionNode lockType,
                    Date expirationDate, boolean inheritance,
                    boolean exclusive) {
        this(locked.getUri(), user.getUri(), lockType.getUri(), expirationDate,
             inheritance, exclusive);
    }
    
    
    /**
     * Constructor.
     *
     * @param objectUri Locked object uri
     * @param subjectUri Lock owner uri
     * @param typeUri Lock type uri
     * @param expirationDate Date of expiration of the lock
     * @param inheritance True if lock is inheritable
     */
    public NodeLock(String objectUri, String subjectUri, String typeUri,
                    Date expirationDate, boolean inheritance) {
        this(objectUri, subjectUri, typeUri, expirationDate,
             inheritance, true);
    }
    
    
    /**
     * Constructor.
     *
     * @param objectUri Locked object uri
     * @param subjectUri Lock owner uri
     * @param typeUri Lock type uri
     * @param expirationDate Date of expiration of the lock
     * @param inheritance True if lock is inheritable
     */
    public NodeLock(String objectUri, String subjectUri, String typeUri,
                    Date expirationDate, boolean inheritance,
                    boolean exclusive) {
        this.objectUri = objectUri;
        this.subjectUri = subjectUri;
        this.typeUri = typeUri;
        this.expirationDate = expirationDate;
        this.inheritance = inheritance;
        this.lockId = objectUri.hashCode() + "_" + subjectUri.hashCode() + "_"
            + typeUri.hashCode() + "_" + expirationDate.getTime() + "_"
            + (new Date()).getTime();
        this.lockId =
            md5Encoder.encode(md5Helper.digest(this.lockId.getBytes()));
        this.exclusive = exclusive;
    }
    
    
    /**
     * Constructor.
     * Creates a new lock linked to another given lock.
     *
     * @param lock Linked lock
     */
    public NodeLock(NodeLock lock, String typeUri) {
        this(lock.getLockId(), lock.getObjectUri(), lock.getSubjectUri(),
             typeUri, lock.getExpirationDate(), lock.isInheritable(),
             lock.isExclusive());
    }
    
    
    /**
     * Constructor.
     *
     * @param objectUri Locked object uri
     * @param subjectUri Lock owner uri
     * @param typeUri Lock type uri
     * @param expirationDate Date of expiration of the lock
     * @param inheritance True if lock is inheritable
     */
    public NodeLock(String lockId, String objectUri, String subjectUri,
                    String typeUri, Date expirationDate, boolean inheritance,
                    boolean exclusive) {
        this.objectUri = objectUri;
        this.subjectUri = subjectUri;
        this.typeUri = typeUri;
        this.expirationDate = expirationDate;
        this.inheritance = inheritance;
        this.lockId = lockId;
        this.exclusive = exclusive;
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Locked object.
     */
    protected String objectUri;
    
    
    /**
     * User who is the lock owner.
     */
    protected String subjectUri;
    
    
    /**
     * Lock type.
     */
    protected String typeUri;
    
    
    /**
     * Expiration date of the lock.
     */
    protected Date expirationDate;
    
    
    /**
     * Lock is inheritable.
     */
    protected boolean inheritance;
    
    
    /**
     * Lock is shared.
     */
    protected boolean exclusive;
    
    
    /**
     * Lock Id.
     */
    protected String lockId;
    
    
    /**
     * MD5 message digest provider.
     */
    protected static MessageDigest md5Helper;


    /**
     * The MD5 helper object for this class.
     */
    protected static final MD5Encoder md5Encoder = new MD5Encoder();


    // ------------------------------------------------------------ Initializer
    
    
    static {
        
        // Load the MD5 helper used to calculate signatures.
        try {
            md5Helper = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new IllegalStateException();
        }
        
    }
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Locked object accessor.
     *
     * @return String Locked subject uri
     */
    public String getObjectUri() {
        return this.objectUri;
    }
    
    
    /**
     * Locked object mutator.
     *
     * @param objectUri Locked subject
     */
    void setObjectUri(String objectUri) {
        this.objectUri = objectUri;
    }
    
    
    /**
     * Lock owner uri accessor.
     *
     * @return String Lock owner
     */
    public String getSubjectUri() {
        return this.subjectUri;
    }
    
    
    /**
     * Lock owner uri mutator.
     *
     * @param subjectUri Lock owner
     */
    void setSubjectUri(String subjectUri) {
        this.subjectUri = subjectUri;
    }
    
    
    /**
     * Lock type uri accessor.
     *
     * @return String Lock type uri
     */
    public String getTypeUri() {
        return this.typeUri;
    }
    
    
    /**
     * Lock type mutator.
     *
     * @param typeUri Lock type
     */
    void setTypeUri(String typeUri) {
        this.typeUri = typeUri;
    }
    
    
    /**
     * Expiration date accessor.
     *
     * @return Date Exipiration date
     */
    public Date getExpirationDate() {
        return expirationDate;
    }
    
    
    /**
     * Expiration date mutator.
     *
     * @param expirationDate Expiration date
     */
    void setExpirationDate(Date expirationDate) {
        this.expirationDate = expirationDate;
    }
    
    
    /**
     * Expiration test.
     *
     * @return boolean True if this lock has expired
     */
    public boolean hasExpired() {
        Date currentTime = new Date();
        return (expirationDate.before(new Date()));
    }
    
    
    /**
     * Inheritance flag accessor.
     *
     * @return boolean True if token is inheritable
     */
    public boolean isInheritable() {
        return inheritance;
    }
    
    
    /**
     * Inheritance flag mutator.
     *
     * @param inheritance Inheritance flag
     */
    void setInheritable(boolean inheritance) {
        this.inheritance = inheritance;
    }
    
    
    /**
     * Exclusive flag accessor.
     *
     * @return boolean True if token is exclusive
     */
    public boolean isExclusive() {
        return exclusive;
    }
    
    
    /**
     * Exclusive flag accessor.
     *
     * @return boolean True if token is shared (ie, not exclusive)
     */
    public boolean isShared() {
        return !exclusive;
    }
    
    
    /**
     * Exclusive flag mutator.
     *
     * @param exclusive Exclusive flag
     */
    void setExclusive(boolean exclusive) {
        this.exclusive = exclusive;
    }
    
    
    /**
     * Get lock identifier.
     *
     * @return String lock id
     */
    public String getLockId() {
        return lockId;
    }
    
    
    // --------------------------------------------------------- Object Methods
    
    
    /**
     * Clone.
     *
     * @return Object clone
     */
    public NodeLock cloneObject() {
        NodeLock result = null;
        try {
            result = (NodeLock) super.clone();
        } catch(CloneNotSupportedException e) {
            e.printStackTrace();
        }
        return result;
    }
    
    
    /**
     * Equals.
     *
     * @param obj Object to test
     * @return boolean True if the two object are equal :
     * <li>obj is of type SlidePermission and is not null</li>
     * <li>All three Uris are equal</li>
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if ((obj != null) && (obj instanceof NodeLock)) {
            NodeLock lock = (NodeLock) obj;
            result = this.getLockId().equals(lock.getLockId());
        }
        return result;
    }
    
    
    /**
     * Validate.
     *
     * @param expectedUri Uri
     */
    public void validate(String expectedUri) {
        
        if (objectUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".nullObjectUri"));
        
        if (!objectUri.equals(expectedUri))
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".incorrectObjectUri"));
        
        if (subjectUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".nullSubjectUri"));
        
        if (typeUri == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".nullTypeUri"));
        
        if (expirationDate == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".nullExpirationDate"));
        
        if (lockId == null)
            throw new ObjectValidationFailedException
                (expectedUri, Messages.message
                 (NodeLock.class.getName() + ".nullLockId"));
        
    }
    
    
}
