/*
 * $Header: /home/cvspublic/jakarta-slide/src/stores/slidestore/file/FileDescriptorsStoreNoVersioning.java,v 1.6 2001/05/16 12:09:00 juergen Exp $
 * $Revision: 1.6 $
 * $Date: 2001/05/16 12:09:00 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package slidestore.file;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.io.*;
import org.apache.slide.common.*;
import org.apache.slide.store.*;
import org.apache.slide.security.*;
import org.apache.slide.structure.*;
import org.apache.slide.content.*;

/**
 * Filesystem implementation of Store.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.6 $
 */
public class FileDescriptorsStoreNoVersioning extends AbstractSimpleService
    implements RevisionDescriptorsStore, RevisionDescriptorStore, 
    SecurityStore, NodeStore {
    
    
    // -------------------------------------------------------------- Constants
    
    
    // Object types
    
    protected static final int SUBJECT = 0;
    protected static final int ACTION = 1;
    protected static final int LINK = 2;
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Path from which files are stored.
     */
    private String rootpath;
    
    
    // ---------------------------------------------------- ServiceImpl Methods
    
    
    /**
     * Read parameters.
     * 
     * @param parameters Hashtable containing the parameters' name 
     * and associated value
     */
    public synchronized void setParameters(Hashtable parameters)
        throws ServiceParameterErrorException, 
        ServiceParameterMissingException {
        // A parameter will tell were serialization files are.
        rootpath = (String) parameters.get("rootpath");
        if (rootpath == null) {
            // Default is that files are stored starting in the 
            // current directory
            rootpath = "";
        }
    }
    
    
    /**
     * Connects to store.
     * 
     * @exception VersionException
     */
    public synchronized void connect()
        throws ServiceConnectionFailedException {
    }
    
    
    /**
     * Disconnects from content store.
     * 
     * @exception ServiceDisconnectionFailedException
     */
    public synchronized void disconnect()
        throws ServiceDisconnectionFailedException {
    }
    
    
    /**
     * Initializes content store.
     * 
     * @exception ServiceInitializationFailedException Throws an exception 
     * if the store has already been initialized before
     */
    public synchronized void initialize(NamespaceAccessToken token)
        throws ServiceInitializationFailedException {
        try {
            File baseDir = new File(rootpath);
            baseDir.mkdirs();
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceInitializationFailedException(this, e);
        }
    }
    
    
    /**
     * Deletes content store.
     * 
     * @exception ServiceResetFailedException
     */
    public void reset()
        throws ServiceResetFailedException {
    }
    
    
    /**
     * This function tells whether or not the service is connected.
     * 
     * @return boolean true if we are connected
     * @exception ServiceAccessException Service access error
     */
    public boolean isConnected() 
        throws ServiceAccessException {
        return true;
    }
    
    
    /**
     * Does nothing here.
     * 
     * @exception ServiceAccessException Unspecified service access error
     */
    public synchronized void commit()
        throws ServiceAccessException {
    }
    
    
    // ------------------------------------------------------ NodeStore Methods
    
    
    /**
     * Retrive an object.
     * 
     * @param uri Uri of the object we want to retrieve
     * @exception ServiceAccessException Error accessing the Data Source
     * @exception ObjectNotFoundException The object to retrieve was not found
     */
    public ObjectNode retrieveObject(Uri uri)
        throws ServiceAccessException, ObjectNotFoundException {
        
        int objectType = SUBJECT;
        String linkTo = null;
        ObjectNode result = null;
        Vector childrenVector = new Vector();
        Vector linksVector = new Vector();
        String relative = uri.getRelative();
        if (relative.equals(""))
            relative = "/";
        
        try {
            
            
            File file = new File(rootpath + relative + ".structure");
            File parentFile = new File(file.getParent());
            if ((parentFile != null) && (!parentFile.exists())) {
                parentFile.mkdirs();
            }
            
            FileReader reader = new FileReader(file);
            BufferedReader bfReader = new BufferedReader(reader);
            
            String type = bfReader.readLine();
            if (type != null) {
                objectType = Integer.parseInt(type);
            }
            if (objectType == LINK) {
                linkTo = bfReader.readLine();
            }
            
            bfReader.close();
            
        } catch (FileNotFoundException e) {
            //e.printStackTrace();
            // Silent exception, except if we're on the root path
            if (relative.equals("/"))
                throw new ObjectNotFoundException(uri);
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
        // Retrieving the children list
        
        try {
            
            File file = new File(rootpath + uri.getRelative());
            if (file.isDirectory()) {
                
                String[] children = file.list();
                
                // Now each of these path need to be converted
                for (int i=0; i < children.length; i++) {
                    String current = children[i].replace('\\','/');
                    int pos = current.indexOf(uri.toString());
                    
                    // DEBUG
                    System.out.println("Adding child :" 
                                       + current.substring(pos + uri.toString().length()));
                    // END DEBUG
                    childrenVector.addElement
                        (current.substring(pos + uri.toString().length()));
                }
                
            }
            
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
        switch (objectType) {
        case SUBJECT :
            result = new SubjectNode(uri.toString(), childrenVector, 
                                     linksVector);
            break;
        case ACTION :
            result = new ActionNode(uri.toString(), childrenVector, 
                                    linksVector);
            break;
        case LINK :
            result = new LinkNode(uri.toString(), childrenVector, 
                                  linksVector, linkTo);
            break;
        }
        
        return result;
    }
    
    
    /**
     * Update an object.
     * 
     * @param object Object to update
     * @exception ServiceAccessException Error accessing the Data Source
     * @exception ObjectNotFoundException The object to update was not found
     */
    public void storeObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException {
    }
    
    
    /**
     * Create a new object.
     * 
     * @param object ObjectNode
     * @param uri Uri of the object we want to create
     * @exception ServiceAccessException Error accessing the Data Source
     * @exception ObjectAlreadyExistsException An object already exists 
     * at this Uri
     */
    public void createObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectAlreadyExistsException {
    }
    
    
    /**
     * Remove an object.
     * 
     * @param object Object to remove
     * @exception ServiceAccessException Error accessing the Data Source
     * @exception ObjectNotFoundException The object to remove was not found
     */
    public void removeObject(Uri uri, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException {
    }
    
    
    // --------------------------------------- RevisionDescriptorsStore Methods
    
    
    /**
     * Retrieve the revisions informations of an object.
     * 
     * @param uri Uri
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor 
     * was not found
     */
    public NodeRevisionDescriptors retrieveRevisionDescriptors(Uri uri)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        
        NodeRevisionNumber initialRevision = new NodeRevisionNumber();
        Hashtable workingRevisions = new Hashtable();
        Hashtable latestRevisionNumbers = new Hashtable();
        Hashtable branches = new Hashtable();
        boolean isVersioned = false;
        
        return new NodeRevisionDescriptors(uri.toString(), initialRevision,
                                           workingRevisions, 
                                           latestRevisionNumbers,
                                           branches, isVersioned);
    }
    
    
    /**
     * Create a new revision information object.
     * 
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptors
     * @exception ServiceAccessException Service access error
     */
    public void createRevisionDescriptors
        (Uri uri, NodeRevisionDescriptors revisionDescriptors)
        throws ServiceAccessException {
    }
    
    
    /**
     * Update revision information.
     * 
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptors
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor 
     * was not found
     */
    public void storeRevisionDescriptors
        (Uri uri, NodeRevisionDescriptors revisionDescriptors)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
    }
    
    
    /**
     * Remove revision information.
     * 
     * @param uri Uri
     * @exception ServiceAccessException Service access error
     */
    public void removeRevisionDescriptors(Uri uri)
        throws ServiceAccessException {
    }
    
    
    // ---------------------------------------- RevisionDescriptorStore Methods
    
    
    /**
     * Retrieve an individual object's revision descriptor.
     * 
     * @param Uri uri
     * @param revisionNumber Node revision number
     */
    public NodeRevisionDescriptor retrieveRevisionDescriptor
        (Uri uri, NodeRevisionNumber revisionNumber)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
        return null;
    }
    
    
    /**
     * Create a new revision descriptor.
     * 
     * @param uri Uri
     * @param revisionDescriptor Node revision descriptor
     * @exception ServiceAccessException Service access error
     */
    public void createRevisionDescriptor
        (Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException {
    }
    
    
    /**
     * Update a revision descriptor.
     * 
     * @param uri Uri
     * @param revisionDescriptors Node revision descriptor
     * @exception ServiceAccessException Service access error
     * @exception RevisionDescriptorNotFoundException Revision descriptor 
     * was not found
     */
    public void storeRevisionDescriptor
        (Uri uri, NodeRevisionDescriptor revisionDescriptor)
        throws ServiceAccessException, RevisionDescriptorNotFoundException {
    }
    
    
    /**
     * Remove a revision descriptor.
     * 
     * @param uri Uri
     * @param revisionNumber Revision number
     * @exception ServiceAccessException Service access error
     */
    public void removeRevisionDescriptor(Uri uri, NodeRevisionNumber number)
        throws ServiceAccessException {
    }
    
    
    // -------------------------------------------------- SecurityStore Methods
    
    
    /**
     * Grant a new permission.
     * 
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Data Source
     */
    public void grantPermission(Uri uri, NodePermission permission)
        throws ServiceAccessException {
        
        try {
            
            File file = new File(rootpath + uri + ".acl");
            File parentFile = new File(file.getParent());
            if ((parentFile != null) && (!parentFile.exists())) {
                parentFile.mkdirs();
            }
            
            FileWriter writer = new FileWriter(file);
            PrintWriter printer = new PrintWriter(writer);
            
            Enumeration oldPermissions = enumeratePermissions(uri);
            
            while (oldPermissions.hasMoreElements()) {
                NodePermission currentPermission = 
                    (NodePermission) oldPermissions.nextElement();
                printer.println(currentPermission.getSubjectUri());
                printer.println(currentPermission.getActionUri());
                printer.println(currentPermission.isInheritable());
                printer.println(currentPermission.isNegative());
            }
            
            printer.println(permission.getSubjectUri());
            printer.println(permission.getActionUri());
            printer.println(permission.isInheritable());
            printer.println(permission.isNegative());
            
            printer.close();
            
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
    }
    
    
    /**
     * Revoke a permission.
     * 
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Data Source
     */
    public void revokePermission(Uri uri, NodePermission permission)
        throws ServiceAccessException {
        
        try {
            
            File file = new File(rootpath + uri + ".acl");
            File parentFile = new File(file.getParent());
            if ((parentFile != null) && (!parentFile.exists())) {
                parentFile.mkdirs();
            }
            
            FileWriter writer = new FileWriter(file);
            PrintWriter printer = new PrintWriter(writer);
            
            Enumeration oldPermissions = enumeratePermissions(uri);
            
            while (oldPermissions.hasMoreElements()) {
                NodePermission currentPermission = 
                    (NodePermission) oldPermissions.nextElement();
                if (!currentPermission.equals(permission)) {
                    printer.println(currentPermission.getSubjectUri());
                    printer.println(currentPermission.getActionUri());
                    printer.println(currentPermission.isInheritable());
                    printer.println(currentPermission.isNegative());
                }
            }
            
            printer.close();
            
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
    }
    
    
    /**
     * Revoke all the permissions on an object.
     * 
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Data Source
     */
    public void revokePermissions(Uri uri)
        throws ServiceAccessException {
        
        try {
            
            File file = new File(rootpath + uri + ".acl");
            File parentFile = new File(file.getParent());
            if ((parentFile != null) && (!parentFile.exists())) {
                parentFile.mkdirs();
            }
            
            FileWriter writer = new FileWriter(file);
            PrintWriter printer = new PrintWriter(writer);
            
            printer.println();
            
            printer.close();
            
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
    }
    
    
    /**
     * Enumerate permissions on an object.
     * 
     * @param permission Permission we want to create
     * @exception ServiceAccessException Error accessing the Data Source
     */
    public Enumeration enumeratePermissions(Uri uri)
        throws ServiceAccessException {
        
        Vector result = new Vector();
        
        try {
            
            File file = new File(rootpath + uri + ".acl");
            File parentFile = new File(file.getParent());
            if ((parentFile != null) && (!parentFile.exists())) {
                parentFile.mkdirs();
            }
            
            FileReader reader = new FileReader(file);
            BufferedReader bfReader = new BufferedReader(reader);
            
            NodePermission permission = null;
            
            String subject = null;
            String action = null;
            String inheritable = null;
            String negative = null;
            
            while (true) {
                
                subject = bfReader.readLine();
                if ((subject == null) || (subject.equals("")))
                    break;
                action = bfReader.readLine();
                if ((action == null) || (action.equals("")))
                    break;
                inheritable = bfReader.readLine();
                if ((inheritable == null) || (inheritable.equals("")))
                    break;
                negative = bfReader.readLine();
                if ((negative == null) || (negative.equals("")))
                    break;
                
                permission = 
                    new NodePermission(uri.toString(), subject, action,
                                        Boolean.getBoolean(inheritable),
                                        Boolean.getBoolean(negative));
                
                result.addElement(permission);
                
            }
            
            bfReader.close();
            
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            // Silent exception
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceAccessException(this, e.getMessage());
        }
        
        return result.elements();
        
    }
    
    
}
