/*
 * $Header: /home/cvspublic/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/methods/DeleteMethod.java,v 1.6 2001/05/01 21:28:02 remm Exp $
 * $Revision: 1.6 $
 * $Date: 2001/05/01 21:28:02 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */
 
package org.apache.webdav.lib.methods;

import org.apache.commons.httpclient.State;
import org.apache.commons.httpclient.Header;


/**
 * DELETE Method.  The delete method can be sent to either a collection or
 * non-collection resource.  If a delete is sent to a collection, then all
 * members of that collection are deleted.
 *
 * <p>   Deletes can fail because of permission problems or if a resource is
 * currently locked.
 *
 * <p>   A typical request/response pair might look like this:
 *
 * <h3>Request</h3>
 * <pre>
 * DELETE /container/ HTTP/1.1
 * Host: www.foo.bar
 * </pre>
 *
 * <h3>Response</h3>
 * <pre>
 * HTTP/1.1 207 Multi-Status
 * Content-Type: text/xml; charset="utf-8"
 * Content-Length: xxxx
 * &lt;?xml version="1.0" encoding="utf-8" ?&gt;
 *   &lt;d:multistatus xmlns:d="DAV:"&gt;
 *   &lt;d:response&gt;
 *     &lt;d:href&gt;http://www.foo.bar/container/resource3&lt;/d:href&gt;
 *     &lt;d:status&gt;HTTP/1.1 423 Locked&lt;/d:status&gt;
 *   &lt;/d:response&gt;
 * &lt;/d:multistatus&gt;
 * </pre>
 *
 * <p>   In this example, the delete failed because one of the members was
 * locked.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @author <a href="mailto:bcholmes@apache.org">B.C. Holmes</a>
 */
public class DeleteMethod
    extends XMLResponseMethodBase {


    // ----------------------------------------------------------- Constructors


    /**
     * Method constructor.
     */
    public DeleteMethod() {
        name = "DELETE";
    }


    /**
     * Method constructor.
     */
    public DeleteMethod(String path) {
        super(path);
        name = "DELETE";
    }


    // --------------------------------------------------- WebdavMethod Methods

}
